#include <sys/types.h>
#include <sys/time.h>
#include <math.h>

#ifdef _SPE_

#include "squid.h"

// helper functions
struct timeval addTime(struct timeval t1, struct timeval t2) {
  struct timeval res;
  res.tv_sec = t1.tv_sec + t2.tv_sec;
  res.tv_usec = t1.tv_usec + t2.tv_usec;
  if (res.tv_usec >= 1000000) {
    res.tv_usec -= 1000000;
    res.tv_sec++;
  }
  return res;
}

struct timeval subTime(struct timeval t1, struct timeval t2) {
  struct timeval res;
  res.tv_sec = t1.tv_sec - t2.tv_sec;
  res.tv_usec = t1.tv_usec - t2.tv_usec;
  if (res.tv_usec < 0) {
    res.tv_usec += 1000000;
    res.tv_sec--;
  }
  return res;
}

struct timeval findRatio(struct timeval total, double ratio) {
  struct timeval avg;
  double val;
  val = (total.tv_sec*1000000.0 + total.tv_usec)*ratio;
  avg.tv_sec = val/1000000;
  avg.tv_usec = (val-(avg.tv_usec*1000000));
  return avg;
}

int getClientIndex(const char *client_addr) {
	int i;
	for (i=0; i<SPEnumberOfClients; i++) {
		if (!strcmp(client_addr, SPEclients[i]))
			break;
	}
	return i;
}

void SPErecordCurrentTime(struct timeval *tm) {
	/* this records the current time;
   * called from comm_select.c:comm_select(), comm_select:comm_poll() 
	 * and SPEstartReceive()
   */

	gettimeofday(tm, NULL);
}

void SPEstartReceive(StoreEntry *e) {
	/* called when an object transfer is started;
	 * e is the object being fetched
	 * called from http.c:httpSendComplete()
	 */

	SPErecordCurrentTime(&(e->lastChunkRecvTime));
	debug(11, 5) ("SPEstartReceive: '%s': starting at %u:%u\n", storeUrl(e), 
								e->lastChunkRecvTime.tv_sec, e->lastChunkRecvTime.tv_usec);
	e->numberOfChunks = 0;
}

void SPEreceiveChunk(StoreEntry *e) {
	/* called when a chunk of response arrives;
	 * e is the object being fetched
	 * called from store.c:storeAppend()
	 */

	struct timeval tm;
	MemObject *mem = e->mem_obj;

	tm = subTime(SPEselectReturnTime, e->lastChunkRecvTime);
	debug(11, 5) ("SPEreceiveChunk: '%s': len:%d,select:%u:%u,chunks=%d,"
								"last:%u:%u,total:%u:%u,diff:%u:%u,release=%s\n", 
								storeUrl(e), mem->inmem_hi,
								SPEselectReturnTime.tv_sec, SPEselectReturnTime.tv_usec,
								e->numberOfChunks, e->lastChunkRecvTime.tv_sec,
								e->lastChunkRecvTime.tv_usec,
								e->totalResponseTime.tv_sec,
								e->totalResponseTime.tv_usec,
								tm.tv_sec,tm.tv_usec,
								EBIT_TEST(e->flags, RELEASE_REQUEST)? "YES":"NO");
	if (mem->inmem_hi == 0) {
		// this is the first chunk
		e->initialResponseTime = tm;
		e->totalResponseTime.tv_sec = e->totalResponseTime.tv_usec = 0;
	}
	else {
		e->totalResponseTime = addTime(e->totalResponseTime, tm);
	}
	e->lastChunkRecvTime = SPEselectReturnTime;
	e->numberOfChunks++;
}

void SPEreceiveComplete(StoreEntry *e) {
	/* called when an object is completely fetched
	 * e is the object fetched
	 * called from http.c:httpReadReply() in two places
	 */ 
	 debug(11, 0) ("SPEreceiveComplete: '%s': conn:%u:%u init:%u:%u "
								 "total:%u:%u size:%d, chunks:%d release=%s\n", 
								 storeUrl(e), e->connectTime.tv_sec,
								 e->connectTime.tv_usec,
								 e->initialResponseTime.tv_sec,
								 e->initialResponseTime.tv_usec,
								 e->totalResponseTime.tv_sec, 
								 e->totalResponseTime.tv_usec,
								 e->mem_obj->inmem_hi, e->numberOfChunks,
								 EBIT_TEST(e->flags, RELEASE_REQUEST)? "YES":"NO");
	e->totalSize = e->mem_obj->inmem_hi;
}

#endif
